import sys


class Terminal(object):
    COLOR = {
        'red': '31',
        'green': '32',
        'blue': '34',
        'gray': '37'
    }
    NORMAL_INTENSITY = '1'
    BRIGHT_INTENSITY = '2'
    ESCAPE = '\x1b['
    RESET = '0'
    SEPARATOR = ';'
    COLOR_CODE = 'm'
    CLEAR_RIGHT_CODE = 'K'

    @classmethod
    def set_color(cls, color):
        """
        color: str - color definition string
        """
        mod = Terminal.NORMAL_INTENSITY
        if color.startswith('bright'):
            mod = Terminal.BRIGHT_INTENSITY
            color = color[len('bright'):]
        color_code = Terminal.COLOR[color]

        sys.stdout.write(cls.ESCAPE + color_code + cls.SEPARATOR + mod
                         + cls.COLOR_CODE)

    @classmethod
    def reset_color(cls):
        sys.stdout.write(cls.ESCAPE + cls.RESET + cls.COLOR_CODE)

    @classmethod
    def clear_right(cls):
        sys.stdout.write(cls.ESCAPE + cls.CLEAR_RIGHT_CODE)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

# Integration between the jstests harness and `WptreportFormatter`.
#
# `WptreportFormatter` uses the data format specified in
# <https://firefox-source-docs.mozilla.org/mozbase/mozlog.html>.

from time import time

from wptrunner.formatters.wptreport import WptreportFormatter


class WptreportHandler(object):
    def __init__(self, out):
        """
        Initialize the WptreportHandler handler.

        :param str out: path to a file to write output to.
        """
        self.out = out
        self.formatter = WptreportFormatter()

    def suite_start(self):
        """
        Produce the "suite_start" message at the present time.
        """
        self.formatter.suite_start({
            "time": time(),
            "run_info": {},
        })

    def suite_end(self):
        """
        Produce the "suite_end" message at the present time and write the
        results to the file path given in the constructor.
        """
        result = self.formatter.suite_end({
            "time": time(),
        })
        with open(self.out, "w") as fp:
            fp.write(result)

    def test(self, result, duration):
        """
        Produce the "test_start", "test_status" and "test_end" messages, as
        appropriate.

        :param dict result: a dictionary with the test results. It should
                            include the following keys:
                            * "name": the ID of the test;
                            * "status": the actual status of the whole test;
                            * "expected": the expected status of the whole test;
                            * "subtests": a list of dicts with keys "test",
                              "subtest", "status" and "expected".
        :param float duration: the runtime of the test
        """
        testname = result["name"]

        end_time = time()
        start_time = end_time - duration

        self.formatter.test_start({
            "test": testname,
            "time": start_time,
        })

        for subtest in result["subtests"]:
            self.formatter.test_status(subtest)

        self.formatter.test_end({
            "test": testname,
            "time": end_time,
            "status": result["status"],
            "expected": result["expected"],
        })
